@@ -10,7 +10,7 @@ from pathlib import Path
 from typing import List
 
 import humanize
-from discord import Emoji, Guild, HTTPException, Thread, User, abc
+from discord import Emoji, Guild, HTTPException, User
 from red_commons.logging import RedTraceLogger, getLogger
 from redbot.core import commands, data_manager
 from redbot.core.bot import Red
@@ -20,6 +20,7 @@ from tidegear import chat_formatting as cf
 from tidegear.constants import ALLOWED_EMOJI_EXTENSIONS, MAX_EMOJI_FILESIZE
 from tidegear.exceptions import ShowToEndUserError
 from tidegear.metadata import CogMetadata
+from tidegear.types import GuildChannel
 from tidegear.utils import send_error
 from tidegear.version import meta
 
@@ -340,7 +341,7 @@ class Cog(commands.Cog):
         return guild
 
     @staticmethod
-    async def get_or_fetch_channel(guild: Guild, channel_id: int) -> abc.GuildChannel | Thread:
+    async def get_or_fetch_channel(guild: Guild, channel_id: int) -> GuildChannel:
         """Retrieve a channel or thread from the internal cache, or fetch it if it cannot be found.
 
         Use this sparingly, as the [`fetch_channel`][discord.Guild.fetch_channel] endpoint has a strict ratelimit.
