@@ -6,7 +6,6 @@
 from typing import Any, Callable
 
 import discord
-from redbot.core import commands
 
 from tidegear import chat_formatting as cf
 
@@ -30,21 +29,22 @@ class TidegearError(Exception):
         self.send_error_kwargs.setdefault("ephemeral", True)
 
     async def send(
-        self, messeagable: commands.Context | discord.abc.Messageable, /, func: Callable[[str], str] = cf.error, **kwargs: Any
+        self, messageable: discord.abc.Messageable | discord.Interaction, /, func: Callable[[str], str] = cf.error, **kwargs: Any
     ) -> discord.Message:
         """Send a message with the contents of this error's message.
+        Wraps [`send_error`][tidegear.utils.send_error], and raises everything that function does.
 
         Args:
-            messeagable: The context to send the message to.
+            messageable: The context to send the message to.
             func: The function to use to wrap the message.
-            **kwargs: Additional keyword arguments to pass to [`ctx.send()`][discord.abc.Messeagable.send].
+            **kwargs: Additional keyword arguments to pass to [`Messageable.send()`][discord.abc.Messageable.send].
 
         Returns:
             The sent message.
         """
         from tidegear.utils import send_error  # noqa: PLC0415 # this is here to prevent potential circular imports in the future
 
-        return await send_error(messeagable, content=self.message, func=func, **self.send_error_kwargs | kwargs)
+        return await send_error(messageable, content=self.message, func=func, **self.send_error_kwargs | kwargs)
 
 
 class ShowToEndUserError(TidegearError):
@@ -59,11 +59,11 @@ class ArgumentError(TidegearError):
     """Raised whenever an argument is not passed to a function that expects it, within **kwargs."""
 
 
-class ArgumentTypeError(ArgumentError):
+class ArgumentTypeError(ArgumentError, TypeError):
     """Raised whenever a keyword argument is passed to a function but has the wrong type. Sort of similar to [`TypeError`][]."""
 
 
-class NotFoundError(TidegearError):
+class NotFoundError(TidegearError, LookupError):
     """Raised whenever an operation doing some kind of search or query fails.
     Essentially a [`LookupError`][], but it is a subclass of [`TidegearError`][tidegear.exceptions.TidegearError].
     """
@@ -71,3 +71,7 @@ class NotFoundError(TidegearError):
 
 class ContextError(TidegearError):
     """Raised whenever a command, function, or method is called from a context it is not supposed to be called from."""
+
+
+class UnmetPermissionsError(TidegearError, PermissionError):
+    """Raised whenever a user or the bot does not have the required permissions to do something."""
