@@ -2,7 +2,6 @@
 # SPDX-License-Identifier: MPL-2.0
 
 from importlib.util import find_spec
-from warnings import warn
 
 if not find_spec("redbot_orm"):
     msg = "redbot-orm is not installed, but the `tidegear.sentinel` module was imported! Did you install tidegear with the `sentinel` extra?"
@@ -12,10 +11,10 @@ if not find_spec("class_registry"):
     msg = "phx-class-registry is not installed, but the `tidegear.sentinel` module was imported! Did you install tidegear with the `sentinel` extra?"
     raise ImportError(msg)
 
-from .cog import SentinelCog
-from .db import AbstractPartial, Change, Moderation, PartialChannel, PartialGuild, PartialUser
+from .cog import SentinelCog, Targetable
+from .db import AbstractPartial, Change, Moderation, PartialChannel, PartialGuild, PartialRole, PartialTargetable, PartialUser
 from .exceptions import HandlerError, LoggedHandlerError, UpsertError
-from .type import ModerationMetadataEntry, ModerationType, moderation_type_registry
+from .moderation_type import ModerationMetadataEntry, ModerationType, moderation_type_registry
 
 __all__ = [
     "AbstractPartial",
@@ -31,6 +30,7 @@ __all__ = [
     "HandlerError",
     "LoggedHandlerError",
     "UpsertError",
+    "PartialRole",
+    "PartialTargetable",
+    "Targetable",
 ]
-
-warn("Tidegear Sentinel is still a heavy work-in-progress, and it has been imported!")
