@@ -8,11 +8,12 @@ import logging as py_logging
 import pathlib
 from collections import defaultdict
 from datetime import date, datetime, timedelta
-from typing import Any, Literal, Mapping
+from types import UnionType
+from typing import Any, Callable, Literal, Mapping
 
 from class_registry.base import RegistryKeyError
 from class_registry.registry import ClassRegistry
-from discord import Color, Embed, Forbidden, Guild, Interaction, Member, Message, NotFound, Permissions, Thread, User, abc, ui
+from discord import Color, Embed, Forbidden, Guild, Interaction, Member, Message, NotFound, Permissions, Role, User, abc, ui
 from discord.abc import MISSING
 from discord.ext import tasks
 from discord.utils import utcnow
@@ -29,12 +30,16 @@ from typing_extensions import override
 
 from tidegear import Cog
 from tidegear import chat_formatting as cf
-from tidegear.exceptions import ConfigurationError, ContextError, NotFoundError
-from tidegear.sentinel.db import Change, Moderation, PartialChannel, PartialGuild, PartialUser
-from tidegear.sentinel.exceptions import HandlerError, LoggedHandlerError, NotReadyError
-from tidegear.sentinel.type import ModerationType, moderation_type_registry
+from tidegear.exceptions import ArgumentTypeError, ConfigurationError, ContextError, NotFoundError, UnmetPermissionsError
+from tidegear.sentinel.db import Change, Moderation, PartialChannel, PartialGuild, PartialRole, PartialTargetable, PartialUser
+from tidegear.sentinel.exceptions import HandlerError, LoggedHandlerError, NotReadyError, UnsetError
+from tidegear.sentinel.moderation_type import ModerationType, moderation_type_registry
+from tidegear.types import GuildChannel, GuildMesseagableChannel
 from tidegear.utils import class_overrides_attribute, get_asset_as_file, send_error, set_env, title, truncate_string
 
+Targetable = GuildChannel | User | Member | Role
+"""Valid types for the target argument in the [`sentinel_moderate`][tidegear.sentinel.SentinelCog.sentinel_moderate] function."""
+
 
 class SentinelCog(Cog):
     """The base cog class for cogs that utilize the Sentinel moderation system.
@@ -95,9 +100,10 @@ class SentinelCog(Cog):
         """
         await super().cog_load()
         self._sentinel_logger.verbose("Using the following path to store the Sentinel SQLite database: %s", self._db_path)
+        trace = True if self._sentinel_logger.level <= py_logging.DEBUG else False
         with set_env(key="PICCOLO_CONF", value="tidegear.sentinel.db.piccolo_conf", logger=self._sentinel_logger):
             self.db = await register_cog(
-                cog_instance=self._db_path, tables=[Change, Moderation, PartialChannel, PartialGuild, PartialUser], trace=True
+                cog_instance=self._db_path, tables=[Change, Moderation, PartialChannel, PartialGuild, PartialRole, PartialUser], trace=trace
             )
 
         for moderation_type in self.sentinel_moderation_types:
@@ -238,7 +244,7 @@ class SentinelCog(Cog):
 
 
             @property
-            def moderation_type_migrations(self) -> Mapping[str, ModerationType]:
+            def moderation_type_migrations(self) -> Mapping[str, type[ModerationType]]:
                 return {"warning": Warn}
             ```
         """
@@ -246,8 +252,10 @@ class SentinelCog(Cog):
 
     @property
     def adds_moderation_types(self) -> bool:
-        """Check whether or not a cog class overrides the [`moderation_types`][tidegear.sentinel.SentinelCog.moderation_types] method."""
-        return class_overrides_attribute(child=type(self), parent=SentinelCog, attribute="moderation_types")
+        """Check whether or not a cog class overrides the
+        [`sentinel_moderation_types`][tidegear.sentinel.SentinelCog.sentinel_moderation_types] property.
+        """
+        return class_overrides_attribute(child=type(self), parent=SentinelCog, attribute="sentinel_moderation_types")
 
     def get_sentinel_cogs(self) -> Mapping[str, "SentinelCog"]:
         """Get a mapping of all cogs that are currently loaded that are subclasses of SentinelCog.
@@ -323,12 +331,15 @@ class SentinelCog(Cog):
 
         return result
 
-    async def sentinel_log_channel(self, guild: Guild) -> abc.GuildChannel | Thread | None:
+    async def sentinel_log_channel(self, guild: Guild) -> GuildMesseagableChannel | None:
         """Retrieve the moderation logging channel for a guild.
 
         Args:
             guild: The guild to fetch the logging channel for.
 
+        Raises:
+            ConfigurationError: If
+
         Returns:
             The fetched channel.
         """
@@ -338,6 +349,12 @@ class SentinelCog(Cog):
 
         try:
             channel = await self.get_or_fetch_channel(guild, channel_id)
+            if not isinstance(channel, GuildMesseagableChannel):
+                msg = (
+                    f"Channel {channel.name} ({channel.id}) in guild {guild.name} {(guild.id)} does not support sending messages, "
+                    "and cannot be used as the Sentinel logging channel."
+                )
+                raise ConfigurationError(msg)
             self._sentinel_logger.trace("channel: %s, channel type: %s", channel, type(channel))
             return channel
         except NotFound:
@@ -345,8 +362,8 @@ class SentinelCog(Cog):
 
     async def sentinel_moderate(
         self,
-        ctx: commands.Context | Interaction,
-        target: Member | User | abc.GuildChannel | Thread,
+        ctx: commands.GuildContext | Interaction,
+        target: Targetable,
         moderation_type: ModerationType | str,
         silent: bool = MISSING,
         reason: str | None = None,
@@ -356,9 +373,24 @@ class SentinelCog(Cog):
 
         Checks if the target can be moderated, then calls the handler method of the moderation type specified.
 
+        Info: Event Dispatch
+            This method, upon success, will dispatch an event you can listen to with the ID `sentinel_moderation`.
+            Here is an example of an event listener that consumes this event.
+
+            ```py
+            from tidegear.sentinel import Moderation, SentinelCog
+
+
+            class ExampleCog(SentinelCog):
+                @SentinelCog.listener("sentinel_moderation")
+                async def on_sentinel_moderation(self, moderation: Moderation):
+                    self.logger.debug(moderation)
+            ```
+
         Args:
             ctx: The context of the command.
-                If this is a [`discord.Interaction`][] object, it will be converted to a [`commands.Context`][redbot.core.commands.Context] object.
+                If this is a [`discord.Interaction`][] object, it will be converted to a
+                [`commands.GuildContext`][redbot.core.commands.GuildContext] object.
                 Additionally, if the interaction originated from a context menu, the [`ctx.author`][discord.ext.commands.Context.author] attribute
                 will be overridden to [`interaction.user`][discord.Interaction.user].
             target: The target user or channel to moderate.
@@ -373,26 +405,29 @@ class SentinelCog(Cog):
         if isinstance(moderation_type, str):
             moderation_type = moderation_type_registry[moderation_type.lower()]
 
-        if isinstance(ctx, Interaction):
-            interaction = ctx
-            ctx = await commands.Context.from_interaction(interaction)
-            if isinstance(interaction.command, app_commands.ContextMenu):
-                ctx.author = interaction.user
-
         if not ctx.guild:
             await send_error(ctx, content="Cannot moderate users outside of a guild!", ephemeral=True)
             return None
 
+        if isinstance(ctx, Interaction):
+            interaction = ctx
+            ctx = await commands.GuildContext.from_interaction(interaction)
+            if isinstance(interaction.command, app_commands.ContextMenu):
+                if not isinstance(interaction.user, Member):
+                    await send_error(ctx, content="Are you real?")
+                    return None
+                ctx.author = interaction.user  # pyright: ignore[reportAttributeAccessIssue]
+
         try:
             await self.sentinel_check_moddable(target=target, ctx=ctx, moderation_type=moderation_type)
-        except (PermissionError, ValueError) as err:
-            await send_error(ctx, content=str(err), ephemeral=True)
+        except (UnmetPermissionsError, ContextError) as err:
+            await err.send(ctx)
             return None
 
         try:
             moderation_handler = moderation_type.handler(target=target)
-        except TypeError as err:
-            await send_error(ctx, content=str(err), ephemeral=True)
+        except ArgumentTypeError as err:
+            await err.send(ctx)
             return None
 
         if silent == MISSING:
@@ -402,7 +437,7 @@ class SentinelCog(Cog):
             reason = await self.get_sentinel_type_config_with_fallback(ctx.guild, moderation_type, "default_reason")
 
         try:
-            if not isinstance(target, (abc.GuildChannel, Thread)):
+            if isinstance(target, (User, Member)):
                 kwargs["silent"] = silent if moderation_type.removes_from_guild else True
 
             moderation = await moderation_handler(cog=self, ctx=ctx, target=target, reason=reason, **kwargs)
@@ -431,6 +466,8 @@ class SentinelCog(Cog):
         except ConfigurationError:
             pass
 
+        self._sentinel_logger.verbose("Moderation success: %s", moderation)
+        self.bot.dispatch("sentinel_moderation", moderation)
         return moderation
 
     def sentinel_check_permissions(
@@ -438,7 +475,7 @@ class SentinelCog(Cog):
         user: Member | abc.User | PartialUser,
         required_permissions: Permissions,
         guild: Guild,
-        channel: abc.GuildChannel | Thread | None = None,
+        channel: GuildChannel | None = None,
     ) -> None:
         """Check if a user has a specific permission (or a list of permissions) in a channel.
 
@@ -455,8 +492,8 @@ class SentinelCog(Cog):
                 attribute that is used when `channel` isn't provided.
 
         Raises:
-            TypeError: Raised if `user` does not match this method's typehints.
-            PermissionError: Raised if the permissions check fails. These error messages are safe to send to end users.
+            ArgumentTypeError: Raised if `user` does not match this method's typehints.
+            UnmetPermissionsError: Raised if the permissions check fails. These error messages are safe to send to end users.
         """
         if isinstance(user, Member):
             member = user
@@ -466,7 +503,7 @@ class SentinelCog(Cog):
             member = guild.get_member(user.discord_id)
         else:
             msg = "Unsupported type!"
-            raise TypeError(msg)
+            raise ArgumentTypeError(msg)
 
         resolved_permissions = None
 
@@ -478,7 +515,7 @@ class SentinelCog(Cog):
 
         if not member or not resolved_permissions:
             msg = f"Could not check permissions for {user.mention}!"
-            raise PermissionError(msg)
+            raise UnmetPermissionsError(msg)
 
         if resolved_permissions.administrator:
             return
@@ -504,12 +541,12 @@ class SentinelCog(Cog):
             part = f"permissions{within_channel}, which are" if len(missing) > 1 else f"permission{within_channel}, which is"
 
             msg = f"{who} do not have the {perms_list} {part} required for this action."
-            raise PermissionError(msg)
+            raise UnmetPermissionsError(msg)
 
     async def sentinel_check_moddable(
         self,
-        ctx: commands.Context,
-        target: User | Member | abc.GuildChannel | Thread,
+        ctx: commands.GuildContext,
+        target: Targetable,
         moderation_type: ModerationType,
     ) -> None:
         """Check if the `ctx.author` can moderate the target.
@@ -536,18 +573,14 @@ class SentinelCog(Cog):
                 Used for checking Red's configuration and confirming that the `ctx.author` and the bot have all of the required permissions.
 
         Raises:
-            ValueError: Raised when `ctx` is outside of a guild context, or the bot user is not in the guild that the passed context originates from.
-            PermissionError: Raised if the target cannot be moderated by the `ctx.author`. These error messages are safe to send to end users.
+            ContextError: Raised when the bot user is not in the guild that the passed context originates from.
+            UnmetPermissionsError: Raised if the target cannot be moderated by the `ctx.author`. These error messages are safe to send to end users.
         """
-        if not ctx.guild or not isinstance(ctx.channel, (abc.GuildChannel, Thread)):
-            msg = "check_moddable is not usable outside of a guild context!"
-            raise ValueError(msg)
-
         if not (me := ctx.guild.get_member(ctx.me.id)):
             msg = "The bot user is not in the guild that the passed Context originates from!"
-            raise ValueError(msg)
+            raise ContextError(msg)
 
-        is_channel = isinstance(target, (abc.GuildChannel, Thread))
+        is_channel = isinstance(target, GuildChannel)
 
         self.sentinel_check_permissions(
             user=me, required_permissions=moderation_type.permissions, guild=ctx.guild, channel=target if is_channel else ctx.channel
@@ -564,23 +597,23 @@ class SentinelCog(Cog):
 
             if ctx.author.id == target.id:
                 msg = "You cannot moderate yourself!"
-                raise PermissionError(msg)
+                raise UnmetPermissionsError(msg)
 
             if not is_moderator_admin and target.bot:
                 msg = "You cannot moderate bots!"
-                raise PermissionError(msg)
+                raise UnmetPermissionsError(msg)
 
             if target.guild_permissions.administrator:
                 msg = "You cannot moderate members with the Administrator permission!"
-                raise PermissionError(msg)
+                raise UnmetPermissionsError(msg)
 
             if me.top_role <= target.top_role:
                 msg = "You cannot moderate members with a role higher than the bot!"
-                raise PermissionError(msg)
+                raise UnmetPermissionsError(msg)
 
             if ctx.author.top_role <= target.top_role and await self.sentinel_config.guild(ctx.guild).respect_hierarchy() is True:
                 msg = "You cannot moderate members with a higher role than you!"
-                raise PermissionError(msg)
+                raise UnmetPermissionsError(msg)
 
             if not is_moderator_admin:
                 immune_roles = await self.sentinel_config.guild(target.guild).immune_roles()
@@ -591,7 +624,7 @@ class SentinelCog(Cog):
                     plural = "s" if len(matching) > 1 else ""
                     part = "they are" if len(matching) > 1 else "it is an"
                     msg = f"You cannot moderate members with the {formatted} role{plural}, because {part} immune role{plural}!"
-                    raise PermissionError(msg)
+                    raise UnmetPermissionsError(msg)
 
     async def sentinel_contact_target(
         self,
@@ -625,7 +658,7 @@ class SentinelCog(Cog):
         guild = await (await moderation.guild()).fetch(bot=self.bot)
         target = await target.fetch(fetcher=self.bot, fetch=True)
 
-        if channel := next((ch for ch in guild.channels if isinstance(ch, abc.Messageable)), None):
+        if channel := next((ch for ch in guild.channels if isinstance(ch, GuildMesseagableChannel)), None):
             color = await self.bot.get_embed_color(location=channel)
         else:
             color = Color.red()
@@ -701,33 +734,12 @@ class SentinelCog(Cog):
             The resulting logging message.
         """
         guild = await (await moderation.guild()).fetch(self.bot)
-        if not (log_channel := await self.sentinel_log_channel(guild)) or not isinstance(log_channel, abc.Messageable):
+        if not (log_channel := await self.sentinel_log_channel(guild)) or not isinstance(log_channel, GuildMesseagableChannel):
             msg = "Logging is disabled!"
             raise ConfigurationError(msg)
         embed = await self.sentinel_case_embed(moderation, color=await self.bot.get_embed_color(location=log_channel))
         return await log_channel.send(embed=embed)
 
-    @staticmethod
-    def is_in_guild(partial_object: PartialChannel | PartialUser, guild: Guild) -> bool:
-        """Check if a PartialChannel or PartialUser is part of a discord Guild.
-
-        Args:
-            partial_object: The Partial to check.
-            guild: The guild to check.
-
-        Raises:
-            TypeError: When `partial_object` is an unsupported type.
-
-        Returns:
-            Whether or not the `partial_object` is in the `guild`.
-        """
-        if isinstance(partial_object, PartialChannel):
-            return bool(guild.get_channel_or_thread(partial_object.channel_id))
-        if isinstance(partial_object, PartialUser):
-            return bool(guild.get_member(partial_object.user_id))
-        msg = f"{partial_object!r} is not a supported type! Got: {type(partial_object)!r}"
-        raise TypeError(msg)
-
     async def sentinel_case_embed(self, moderation: Moderation, color: Color | None = None) -> Embed:
         """Create a case embed from a Moderation object.
 
@@ -751,9 +763,6 @@ class SentinelCog(Cog):
         except Exception:
             pass
 
-        target_in_guild = self.is_in_guild(target, guild)
-        moderator_in_guild = self.is_in_guild(moderator, guild)
-
         moderation_type = moderation.type_key
         try:
             moderation_type = moderation.type.name
@@ -767,9 +776,10 @@ class SentinelCog(Cog):
 
         embed.description = (
             f"{cf.bold('Type:')} {title(moderation_type)}\n"
-            f"{cf.bold('Target:')} {target.mention if target_in_guild else cf.inline(text=target.name)} ({cf.inline(text=str(target.discord_id))})\n"
-            f"{cf.bold('Moderator:')} {moderator.mention if moderator_in_guild else cf.inline(text=moderator.name)} "
-            f"({cf.inline(text=str(moderator.discord_id))})\n"
+            f"{cf.bold('Target:')} {target.mention if target.in_guild(guild) else cf.inline(text=target.name)} "
+            f"({cf.inline(str(target.discord_id))})\n"
+            f"{cf.bold('Moderator:')} {moderator.mention if moderator.in_guild(guild) else cf.inline(text=moderator.name)} "
+            f"({cf.inline(str(moderator.discord_id))})\n"
             f"{cf.bold('Timestamp:')} {cf.format_datetime(dt=moderation.timestamp)} "
             f"| {cf.format_datetime(dt=moderation.timestamp, style=cf.TimestampStyle.RELATIVE)}"
         )
@@ -792,7 +802,7 @@ class SentinelCog(Cog):
         try:
             moderation_type = moderation.type
             for metadata in moderation_type.metadata:
-                if value := metadata.fetch_from_moderation(moderation):
+                if value := await metadata.fetch_from_moderation(moderation):
                     embed.description += f"\n{cf.bold(f'{title(metadata.human_name)}:')} {value}"
         except RegistryKeyError:
             pass
@@ -842,11 +852,10 @@ class SentinelCog(Cog):
 
             for change in changes[i : i + fields_per_embed]:
                 moderator = await change.moderator()
-                mod_in_guild = self.is_in_guild(moderator, guild)
 
                 change_str = [
                     (
-                        f"{cf.bold('Moderator:')} {moderator.mention if mod_in_guild else cf.inline(text=moderator.name)} "
+                        f"{cf.bold('Moderator:')} {moderator.mention if moderator.in_guild(guild) else cf.inline(text=moderator.name)} "
                         f"({cf.inline(text=str(moderator.discord_id))})"
                     ),
                     f"{cf.bold('Reason:')} {truncate_string(change.reason, 200)}" if change.reason else "",
@@ -876,9 +885,9 @@ class SentinelCog(Cog):
 
     async def sentinel_history_menu(
         self,
-        ctx: commands.Context,
+        ctx: commands.GuildContext,
         *,
-        targets: list[abc.User | PartialUser | abc.GuildChannel | Thread | PartialChannel] | None = None,
+        targets: list[Targetable | PartialTargetable] | None = None,
         moderators: list[abc.User | PartialUser] | None = None,
         on: date | None = None,
         before: datetime | None = None,
@@ -941,21 +950,39 @@ class SentinelCog(Cog):
 
         user_targets: list[PartialUser] = []
         channel_targets: list[PartialChannel] = []
-        for t in targets or []:
-            if isinstance(t, abc.User):
-                user_targets.append(await PartialUser.upsert(user=t))
-            elif isinstance(t, (abc.GuildChannel, Thread)):
-                channel_targets.append(await PartialChannel.upsert(channel=t))
+        role_targets: list[PartialRole] = []
 
-        if user_targets and channel_targets:
-            where.append(
-                Moderation.target_user_id.id.is_in([user.id for user in user_targets])
-                | Moderation.target_channel_id.id.is_in([channel.id for channel in channel_targets])
-            )
-        elif user_targets:
-            where.append(Moderation.target_user_id.id.is_in([user.id for user in user_targets]))
-        elif channel_targets:
-            where.append(Moderation.target_channel_id.id.is_in([channel.id for channel in channel_targets]))
+        type_map: dict[type | UnionType, tuple[list[Any], Callable[[Any], Any] | None]] = {
+            PartialUser: (user_targets, None),
+            PartialChannel: (channel_targets, None),
+            PartialRole: (role_targets, None),
+            abc.User: (user_targets, PartialUser.upsert),
+            GuildChannel: (channel_targets, PartialChannel.upsert),
+            Role: (role_targets, PartialRole.upsert),
+        }
+
+        for t in targets or []:
+            for target_type, (collection, upsert_fn) in type_map.items():
+                if isinstance(t, target_type):
+                    if upsert_fn is None:
+                        collection.append(t)
+                    else:
+                        collection.append(await upsert_fn(t))
+                    break  # don’t check further types once matched
+
+        clauses = []
+        if user_targets:
+            clauses.append(Moderation.target_user_id.id.is_in([u.id for u in user_targets]))
+        if channel_targets:
+            clauses.append(Moderation.target_channel_id.id.is_in([c.id for c in channel_targets]))
+        if role_targets:
+            clauses.append(Moderation.target_role_id.id.is_in([r.id for r in role_targets]))
+
+        if clauses:
+            condition = clauses[0]
+            for clause in clauses[1:]:
+                condition |= clause
+            where.append(condition)
 
         partial_moderators: list[PartialUser] = []
         for m in moderators or []:
@@ -1022,7 +1049,7 @@ class SentinelCog(Cog):
         elif self.me.avatar:
             icon_url = self.me.avatar.url
 
-        moderation_deltas: list[timedelta] = []
+        moderation_deltas: dict[int, timedelta] = {}
 
         for i in range(0, count, pagesize):
             moderation_before = utcnow()
@@ -1032,26 +1059,28 @@ class SentinelCog(Cog):
             embed.timestamp = timestamp
 
             for moderation in moderations[i : i + pagesize]:
-                target: PartialUser | PartialChannel | None = None
-
-                if moderation.target_user_id.user_id:
-                    target = moderation.target_user_id
-                elif moderation.target_channel_id.channel_id:
-                    target = moderation.target_channel_id
-
-                if not target:
-                    self._internal_logger.error("Moderation %i does not have a target!", moderation.id)
+                try:
+                    target = await moderation.target()
+                except UnsetError:
+                    self._internal_logger.exception("Moderation %i does not have a target!", moderation.id)
+                    continue
+                except NotFoundError:
+                    self._internal_logger.exception("Could not find target for moderation %i!", moderation.id)
                     continue
 
-                moderator: PartialUser = moderation.moderator_id
-
-                target_in_guild = self.is_in_guild(partial_object=target, guild=ctx.guild)
-                moderator_in_guild = self.is_in_guild(partial_object=moderator, guild=ctx.guild)
+                try:
+                    moderator = await moderation.moderator()
+                except UnsetError:
+                    self._internal_logger.exception("Moderation %i does not have a moderator!", moderation.id)
+                    continue
+                except NotFoundError:
+                    self._internal_logger.exception("Could not find moderator for moderation %i!", moderation.id)
+                    continue
 
                 value = (
-                    f"{cf.bold(text='Target:')} {target.mention if target_in_guild else cf.inline(text=target.name)} "
+                    f"{cf.bold(text='Target:')} {target.mention if target.in_guild(ctx.guild) else cf.inline(text=target.name)} "
                     f"({cf.inline(text=str(target.discord_id))})\n"
-                    f"{cf.bold(text='Moderator:')} {moderator.mention if moderator_in_guild else cf.inline(text=moderator.name)} "
+                    f"{cf.bold(text='Moderator:')} {moderator.mention if moderator.in_guild(ctx.guild) else cf.inline(text=moderator.name)} "
                     f"({cf.inline(text=str(moderator.discord_id))})\n"
                     f"{cf.bold(text='Reason:')} {cf.inline(text=truncate_string(moderation.reason or 'No reason provided.', 140))}\n"
                     f"{cf.bold(text='Timestamp:')} {cf.format_datetime(dt=moderation.timestamp, style=cf.TimestampStyle.LONG_DATE_AND_TIME)}"
@@ -1081,26 +1110,32 @@ class SentinelCog(Cog):
 
                 if moderation_type:
                     for meta in [meta for meta in moderation_type.metadata if meta.show_in_history]:
-                        if meta_value := meta.fetch_from_moderation(moderation):
+                        if meta_value := await meta.fetch_from_moderation(moderation):
                             value += f"{cf.bold(title(meta.human_name) + ':')} {meta_value}\n"
 
                 embed.add_field(name=f"Case {moderation.id:,} ({title(moderation.type_key)})", value=value, inline=inline)
-            pages.append(embed)
-            moderation_after = utcnow()
-            moderation_deltas.append(moderation_after - moderation_before)
 
-        total_moderation_time = sum(moderation_deltas, timedelta(0))
+                moderation_after = utcnow()
+                moderation_deltas[moderation.id] = moderation_after - moderation_before
+            pages.append(embed)
+
+        total_moderation_time = sum(moderation_deltas.values(), timedelta(0))
         moderation_milliseconds = total_moderation_time.total_seconds() * 1_000
         total_ms = query_construction_milliseconds + query_milliseconds + moderation_milliseconds
         moderation_average = (total_moderation_time / len(moderation_deltas)).total_seconds() * 1_000
-        moderation_highest = max(moderation_deltas, default=timedelta(0)).total_seconds() * 1_000
+        moderation_highest_key, moderation_highest_value = max(
+            moderation_deltas.items(),
+            key=lambda kv: kv[1],
+            default=(0, timedelta(0)),
+        )
+        moderation_highest_milliseconds = moderation_highest_value.total_seconds() * 1_000
 
-        self._internal_logger.trace(
+        self._sentinel_logger.trace(
             f"History query completed in {total_ms:,.3f} ms with {len(moderations):,} moderations retrieved! "
             f"Query construction time: '{query_construction_milliseconds:,.3f} ms' "
             f"Query time: '{query_milliseconds:,.3f} ms' "
             f"Moderation iteration time: '{moderation_milliseconds:,.3f} ms' "
-            f"Longest moderation iteration: '{moderation_highest:,.3f} ms' "
+            f"Longest moderation iteration: 'Moderation {moderation_highest_key:,} with {moderation_highest_milliseconds:,.3f} ms' "
             f"Avg. time per moderation: '{moderation_average:,.3f} ms'"
         )
 
@@ -1166,7 +1201,7 @@ class SentinelCog(Cog):
                     err_num += 1
 
             per_guild_completion_time = utcnow() - time_per_guild
-            self._sentinel_logger.verbose(
+            self._sentinel_logger.trace(
                 "Completed expiry loop for guild '%s' (%s) in %sms with %s successful cases and %s errors.",
                 guild.name,
                 guild.guild_id,
