@@ -11,7 +11,7 @@ from typing import TYPE_CHECKING, Any, Literal, Never, Self, overload
 import orjson
 import rich.repr
 from class_registry.base import RegistryKeyError
-from discord import Guild, Member, Object, Thread, User, abc
+from discord import Guild, Member, Object, Role, User, abc
 from piccolo.columns.column_types import JSON, Boolean, ForeignKey, Integer, Serial, Text, Timestamptz, Varchar
 from piccolo.columns.defaults.timestamptz import TimestamptzNow
 from piccolo.table import Table as BaseTable
@@ -20,7 +20,8 @@ from typing_extensions import override
 
 from tidegear.exceptions import NotFoundError
 from tidegear.sentinel.exceptions import NotReadyError, UnsetError, UpsertError
-from tidegear.sentinel.type import ModerationType, moderation_type_registry
+from tidegear.sentinel.moderation_type import ModerationType, moderation_type_registry
+from tidegear.types import GuildChannel
 
 if TYPE_CHECKING:
     from tidegear.sentinel.cog import SentinelCog
@@ -144,6 +145,11 @@ class AbstractPartial:
         """Return the partial's Discord ID according to the internal database."""
         ...
 
+    @abstractmethod
+    def in_guild(self, guild: Guild) -> bool:
+        """Return whether or not the partial is a member of a given [`discord.Guild`][]."""
+        ...
+
 
 class PartialGuild(Table, AbstractPartial):
     """A model representing a guild stored within the internal database.
@@ -181,6 +187,10 @@ class PartialGuild(Table, AbstractPartial):
     def discord_id(self) -> int:
         return self.guild_id
 
+    @override
+    def in_guild(self, guild: Guild) -> bool:
+        return True
+
     @overload
     async def fetch(self, bot: Red, *, fetch: Literal[True] = ..., upsert: bool = ...) -> Guild: ...
 
@@ -272,6 +282,10 @@ class PartialUser(Table, AbstractPartial):
     def discord_id(self) -> int:
         return self.discord_object.id
 
+    @override
+    def in_guild(self, guild: Guild) -> bool:
+        return bool(guild.get_member(self.discord_id))
+
     @overload
     async def fetch(self, fetcher: Red, *, fetch: Literal[True] = ..., upsert: bool = ...) -> User: ...
     @overload
@@ -378,13 +392,17 @@ class PartialChannel(Table, AbstractPartial):
     def discord_id(self) -> int:
         return self.discord_object.id
 
-    @overload
-    async def fetch(self, bot: Red, *, fetch: Literal[True] = ..., upsert: bool = ...) -> abc.GuildChannel | Thread: ...
+    @override
+    def in_guild(self, guild: Guild) -> bool:
+        return bool(guild.get_channel_or_thread(self.discord_id))
 
     @overload
-    async def fetch(self, bot: Red, *, fetch: Literal[False] = False, upsert: bool = ...) -> abc.GuildChannel | Thread | None: ...
+    async def fetch(self, bot: Red, *, fetch: Literal[True] = ..., upsert: bool = ...) -> GuildChannel: ...
 
-    async def fetch(self, bot: Red, *, fetch: bool = False, upsert: bool = True) -> abc.GuildChannel | Thread | None:
+    @overload
+    async def fetch(self, bot: Red, *, fetch: Literal[False] = False, upsert: bool = ...) -> GuildChannel | None: ...
+
+    async def fetch(self, bot: Red, *, fetch: bool = False, upsert: bool = True) -> GuildChannel | None:
         """Retrieve a GuildChannel or Thread object for this PartialChannel.
 
         Only use this if you need more information than is stored within the database.
@@ -438,7 +456,7 @@ class PartialChannel(Table, AbstractPartial):
         return self._guild_obj
 
     @classmethod
-    async def upsert(cls, channel: abc.GuildChannel | Thread) -> Self:
+    async def upsert(cls, channel: GuildChannel) -> Self:
         """Insert or update a row in the database based on metadata from a GuildChannel or Thread object.
 
         Args:
@@ -450,20 +468,159 @@ class PartialChannel(Table, AbstractPartial):
         Returns:
             (PartialChannel): The resulting PartialChannel object.
         """
-        query = cls.objects(cls.guild_id).where(cls.channel_id == channel.id, cls.guild_id.guild_id == channel.guild.id).first()
+        guild = await PartialGuild.upsert(guild=channel.guild)
+
+        query = cls.objects(cls.all_related()).where(cls.channel_id == channel.id, cls.guild_id.id == guild.id).first()
         if fetched_channel := await query:
             await fetched_channel.update_self(values={cls.last_known_name: channel.name, cls.last_updated: TimestamptzNow().python()})
+            fetched_channel._guild_obj = guild  # noqa: SLF001
             return fetched_channel
 
-        guild = await PartialGuild.upsert(guild=channel.guild)
         await cls.insert(cls(_data={cls.guild_id: guild.id, cls.channel_id: channel.id, cls.last_known_name: channel.name}))
 
         if result := await query:
+            result._guild_obj = guild  # noqa: SLF001
             return result
         msg = "Upsert operation failed!"
         raise UpsertError(msg)
 
 
+class PartialRole(Table, AbstractPartial):
+    """A model representing a role stored within the internal database.
+
+    Attributes: Columns:
+        id: The internal ID of the role within the database. This is NOT the role's Discord ID.
+        guild_id: The internal ID of the guild this role is parented to within the database. This is NOT the guild's Discord ID.
+        role_id: The Discord ID of the role.
+            Please consider using [`.discord_id`][tidegear.sentinel.AbstractPartial.discord_id] instead of this.
+        last_known_name: The name of the role, as of the last time the role was upserted.
+        last_updated: The last time the role was upserted.
+    """
+
+    id: Serial = Serial(index=True, primary_key=True)
+    guild_id: ForeignKey[PartialGuild] = ForeignKey(references=PartialGuild, null=False)
+    role_id: Integer = Integer(index=True)
+    last_known_name: Varchar = Varchar(default="Unknown Role", length=100)
+    last_updated: Timestamptz = Timestamptz(default=datetime.now, null=False)
+
+    @property
+    @override
+    def name(self) -> str:
+        return self.last_known_name
+
+    @property
+    @override
+    def mention(self) -> str:
+        return f"<@&{self.role_id}>"
+
+    @property
+    @override
+    def discord_object(self) -> Object:
+        return Object(id=self.role_id, type=Role)
+
+    @property
+    @override
+    def discord_id(self) -> int:
+        return self.discord_object.id
+
+    @override
+    def in_guild(self, guild: Guild) -> bool:
+        return bool(guild.get_role(self.discord_id))
+
+    @overload
+    async def fetch(self, bot: Red, *, fetch: Literal[True] = ..., upsert: bool = ...) -> Role: ...
+
+    @overload
+    async def fetch(self, bot: Red, *, fetch: Literal[False] = False, upsert: bool = ...) -> Role | None: ...
+
+    async def fetch(self, bot: Red, *, fetch: bool = False, upsert: bool = True) -> Role | None:
+        """Retrieve a Role object for this PartialRole.
+
+        Only use this if you need more information than is stored within the database.
+
+        Args:
+            bot: The bot object to use to retrieve the role.
+            fetch: Whether or not to attempt to fetch the role from Discord's API if the role is not in the internal cache.
+                Avoid using this unless absolutely necessary, as this endpoint is ratelimited and introduces additional runtime cost.
+            upsert: Whether or not to automatically upsert the retrieved role object into the database.
+                This introduces a minimal runtime cost if you're already fetching from the Discord API, and should usually be done.
+
+        Returns:
+            The retrieved role object, or None if `fetch` is `False` and the guild or role is not in the bot's internal cache.
+        """
+        partial_guild = await self.guild()
+        if not (guild := await partial_guild.fetch(bot, fetch=fetch)):
+            return None
+
+        role = guild.get_role(self.role_id)
+        if fetch and not role:
+            role = await guild.fetch_role(self.role_id)
+
+        if upsert and role:
+            await self.upsert(role)
+        return role
+
+    async def guild(self, /, *, cache: bool = True) -> PartialGuild:
+        """Retrieve the [`PartialGuild`][tidegear.sentinel.PartialGuild] that this role belongs to.
+
+        Args:
+            cache: Whether or not to try an internal cache before querying the database.
+                This will have no effect if you haven't ran this method yet on this PartialRole instance.
+
+        Raises:
+            NotFoundError: If the guild tied to this role no longer exists in the database.
+                This should be reported as a bug if it occurs, as there's a foreign key constraint that should prevent this on the table itself.
+
+        Returns:
+            The guild object tied to this role.
+        """
+        if cache and "_guild_obj" in self.__dict__:
+            return self._guild_obj
+
+        if isinstance(self.guild_id, PartialGuild):  # support both initialized PartialGuild objects and raw ints
+            guild = self.guild_id
+        elif not (guild := await PartialGuild.objects(PartialGuild.all_related()).where(PartialGuild.id == self.guild_id).first()):
+            msg = f"No guild exists in the database with id {self.guild_id}!"
+            raise NotFoundError(msg)
+
+        self._guild_obj = guild
+        return self._guild_obj
+
+    @classmethod
+    async def upsert(cls, role: Role) -> Self:
+        """Insert or update a row in the database based on metadata from a Role object.
+
+        Args:
+            role: The role object to upsert.
+
+        Raises:
+            UpsertError: If upserting fails for some reason.
+
+        Returns:
+            (PartialRole): The resulting PartialRole object.
+        """
+        guild = await PartialGuild.upsert(guild=role.guild)
+
+        query = cls.objects(cls.all_related()).where(cls.role_id == role.id, cls.guild_id.guild_id == role.guild.id).first()
+        if fetched_role := await query:
+            await fetched_role.update_self(values={cls.last_known_name: role.name, cls.last_updated: TimestamptzNow().python()})
+            fetched_role._guild_obj = guild  # noqa: SLF001
+            return fetched_role
+
+        await cls.insert(cls(_data={cls.guild_id: guild.id, cls.role_id: role.id, cls.last_known_name: role.name}))
+
+        if result := await query:
+            result._guild_obj = guild  # noqa: SLF001
+            return result
+        msg = "Upsert operation failed!"
+        raise UpsertError(msg)
+
+
+PartialTargetable = PartialChannel | PartialRole | PartialUser
+"""Partial tables that implement [`AbstractPartial`][tidegear.sentinel.AbstractPartial]
+and may be set as targets within [`Moderation`][tidegear.sentinel.Moderation] rows."""
+
+
 class Change(Table):
     """A database model representing a change to a moderation case.
 
@@ -582,11 +739,12 @@ class Moderation(Table):
         target_user_id: The internal ID of the user this moderation case targets within the database.
             **This is not a Discord User ID!**
             Use [`.target()`][tidegear.sentinel.Moderation.target] to get an actual [`PartialUser`][tidegear.sentinel.PartialUser] object.
-            This **or** `target_channel_id` should always be set, but not both.
-        target_channel_id: The internal ID of the user this moderation case targets within the database.
+        target_channel_id: The internal ID of the channel this moderation case targets within the database.
             **This is not a Discord Channel ID!**
             Use [`.target()`][tidegear.sentinel.Moderation.target] to get an actual [`PartialChannel`][tidegear.sentinel.PartialChannel] object.
-            This **or** `target_user_id` should always be set, but not both.
+        target_role_id: The internal ID of the role this moderation case targets within the database.
+            **This is not a Discord Role ID!**
+            Use [`.target()`][tidegear.sentinel.Moderation.target] to get an actual [`PartialRole`][tidegear.sentinel.PartialChannel] object.
         moderator_id: The internal ID of the user this moderation case was created by within the database.
             **This is not a Discord User ID!**
             Use [`.moderator()`][tidegear.sentinel.Moderation.moderator] to get an actual [`PartialUser`][tidegear.sentinel.PartialUser] object.
@@ -608,6 +766,7 @@ class Moderation(Table):
     type_key: Varchar = Varchar(default=None, null=False, db_column_name="type")
     target_user_id: ForeignKey[PartialUser] = ForeignKey(references=PartialUser, null=True, index=True)
     target_channel_id: ForeignKey[PartialChannel] = ForeignKey(references=PartialChannel, null=True, index=True)
+    target_role_id: ForeignKey[PartialRole] = ForeignKey(references=PartialRole, null=True, index=True)
     moderator_id: ForeignKey[PartialUser] = ForeignKey(references=PartialUser, null=False, index=True)
     end_timestamp: Timestamptz = Timestamptz(default=None, null=True, index=True)
     expired: Boolean = Boolean(default=False, null=False)
@@ -748,7 +907,7 @@ class Moderation(Table):
         self._moderator_obj = user
         return user
 
-    async def target(self, /, *, cache: bool = True) -> PartialUser | PartialChannel:
+    async def target(self, /, *, cache: bool = True) -> PartialTargetable:
         """Retrieve the target of this moderation.
 
         Args:
@@ -756,18 +915,17 @@ class Moderation(Table):
                 This will have no effect if you haven't ran this method yet on this Moderation instance.
 
         Raises:
-            NotFoundError: If the [`PartialUser`][tidegear.sentinel.PartialUser] or [`PartialChannel`][tidegear.sentinel.PartialChannel]
-                tied to this moderation no longer exists in the database.
+            NotFoundError: If the target tied to this moderation no longer exists in the database.
                 This should be reported as a bug if it occurs, as there's a foreign key constraint that should prevent this on the table itself.
-            UnsetError: If neither `target_user` or `target_channel` are set on the moderation object.
+            UnsetError: If `target_user`, `target_channel`, and `target_role` are none set on the moderation object.
 
         Returns:
-            The PartialUser or PartialChannel that was targeted by this moderation.
+            A partial object tied to the object that was targeted by this moderation.
         """
         if cache and "_target_obj" in self.__dict__:
             return self._target_obj
 
-        result: PartialChannel | PartialUser | None = None
+        result: PartialTargetable | None = None
 
         if self.target_user_id is not None:
             if isinstance(self.target_user_id, PartialUser):  # support both initialized PartialUser objects and raw ints
@@ -787,8 +945,16 @@ class Moderation(Table):
                 msg = f"Could not find a PartialChannel in the database with id {self.target_channel_id}"
                 raise NotFoundError(msg)
 
+        if not result and self.target_role_id is not None:
+            if isinstance(self.target_role_id, PartialRole):  # support both initialized PartialRole objects and raw ints
+                if not self.target_role_id.is_null():
+                    result = self.target_role_id
+            elif not (result := await PartialRole.objects(PartialRole.all_related()).where(PartialRole.id == self.target_role_id).first()):
+                msg = f"Could not find a PartialRole in the database with id {self.target_role_id}"
+                raise NotFoundError(msg)
+
         if not result:
-            msg = "Neither target_user nor target_channel are set!"
+            msg = f"Moderation {self.id:,} has no target set!"
             raise UnsetError(msg)
 
         self._target_obj = result
