@@ -10,7 +10,7 @@ import warnings
 from contextlib import contextmanager
 from importlib.resources import as_file, files
 from importlib.util import find_spec
-from typing import TYPE_CHECKING, Any, Callable, Generator, Type, TypeVar, overload
+from typing import TYPE_CHECKING, Any, Callable, Generator, TypeVar, overload
 
 import discord
 from discord.utils import MISSING
@@ -19,7 +19,7 @@ from redbot.core import commands
 
 from tidegear import chat_formatting as cf
 from tidegear.constants import FALSE, NONE, TRUE
-from tidegear.exceptions import ArgumentError, ArgumentTypeError
+from tidegear.exceptions import ArgumentError, ArgumentTypeError, ContextError
 from tidegear.metadata import SemVer
 from tidegear.version import version
 
@@ -227,7 +227,7 @@ def title(string: str, /) -> str:
 
 
 async def send_error(
-    messeagable: commands.Context | discord.abc.Messageable,
+    messageable: discord.abc.Messageable | discord.Interaction,
     /,
     content: str | None = None,
     func: Callable[[str], str] = cf.error,
@@ -236,17 +236,26 @@ async def send_error(
     """Send a message with the content wrapped in an error function.
 
     Args:
-        messeagable: The context to send the message to.
+        messageable: The context to send the message to.
         content: The content of the message.
         func: The function to use to wrap the message.
-        **kwargs: Additional keyword arguments to pass to [`ctx.send()`][discord.abc.Messeagable.send].
+        **kwargs: Additional keyword arguments to pass to [`Messageable.send()`][discord.abc.Messageable.send].
+
+    Raises:
+        ContextError: If `messageable` is an [`interaction`][discord.Interaction] and the interaction does not originate from an application command.
 
     Returns:
         The sent message.
     """
+    if isinstance(messageable, discord.Interaction):
+        if messageable.type != discord.InteractionType.application_command:
+            msg = "Invalid interaction passed! Interactions must originate from application commands to be used with 'send_error'."
+            raise ContextError(msg)
+        messageable = await commands.Context.from_interaction(messageable)
+
     if content:
         content = func(content)
-    return await messeagable.send(content=content, **kwargs)
+    return await messageable.send(content=content, **kwargs)
 
 
 @overload
@@ -256,7 +265,7 @@ def get_kwarg(
     argument: str,
     *,
     default: Any = ...,
-    expected_type: Type[_T] | tuple[Type[_T], ...],
+    expected_type: type[_T] | tuple[type[_T], ...],
 ) -> _T: ...
 @overload
 def get_kwarg(
@@ -269,7 +278,7 @@ def get_kwarg(
 ) -> Any: ...
 
 
-def get_kwarg(kwargs: dict[str, Any], /, argument: str, *, default: Any = MISSING, expected_type: Type | tuple[Type, ...] | None = None) -> Any:
+def get_kwarg(kwargs: dict[str, Any], /, argument: str, *, default: Any = MISSING, expected_type: type | tuple[type, ...] | None = None) -> Any:
     """Retrieve a keyword argument from a dictionary with optional type validation.
 
     This function is a strict helper for fetching values from a `kwargs` dictionary.
